﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using XeP3;
using csaWebCommon.Exceptions;
using csaCommon;
using csaWebCommon;
using Infragistics.Web.UI.EditorControls;
using Infragistics.Web.UI.GridControls;
using System.Web.Services;

public partial class Forms_ProjectPlan : csaWebBasePage
{
    #region Properties and State

    public string DisplayParam
    {
        get
        {
            string _DisplayParam = Request.QueryString["Show"];
            if (_DisplayParam == null)
            {
                _DisplayParam = "OwnerPlan";
            }
            if (!csaApplication.CurrentUser.Admin && csaApplication.CurrentUser.Projects[csaApplication.CurrentProject.ProjectID].ProjectManager == 0)
                // they are NOT a manager or admin - so override an "All" setting if they try to select the whole plan...
                _DisplayParam = "OwnerPlan";
            return _DisplayParam;
        }
    }

    private const string SessionState_Owner = "Owner";
    private List<User> _Owners;
    public List<User> Owners
    {
        get
        {
            if (_Owners == null)
                _Owners = (List<User>)HttpContext.Current.Session[SessionState_Owner];
            if (_Owners == null)
                _Owners = UsersManager.GetUsersByProject(csaApplication.CurrentProject.ProjectID, true);
            return _Owners;
        }
        set
        {
            HttpContext.Current.Session[SessionState_Owner] = value;
            _Owners = value;
        }
    }

    private const string SessionState_Status = "Status";
    private List<Lookup> _Status;
    public List<Lookup> Status
    {
        get
        {
            if (_Status == null)
                _Status = (List<Lookup>)HttpContext.Current.Session[SessionState_Status];
            if (_Status == null)
                _Status = LookupManager.GetLookupsByField("tbl_Action", "Status", true, " ");
            return _Status;
        }
        set
        {
            HttpContext.Current.Session[SessionState_Status] = value;
            _Status = value;
        }
    }

    private const string SessionState_ProjectParam = "SessionState_ProjectParam";

    #endregion

    protected void Page_Load(object sender, EventArgs e)
    {
        if (!IsPostBack)
        {
            if (DisplayParam == "All" && ProjectView())
            {
                pnlOwner.Header.Text = string.Format("Display all Actions in {0} Project Plan", csaApplication.CurrentProject.Description);
                cboActionOwner.SelectedValue = "0";
                pnlOwner.Expanded = false;  // is neater if closed
                HttpContext.Current.Session[SessionState_ProjectParam] = DisplayParam;
            }
            else
            {
                // Display owner plan variation on form, user gets no choice:
                cboActionOwner.SelectedValue = csaApplication.CurrentUser.UserID.ToString();
                pnlOwner.Header.ExpansionIndicator.Width = Unit.Pixel(0);
                pnlOwner.Expanded = false;
                HttpContext.Current.Session[SessionState_ProjectParam] = "OwnerPlan";
            }
        }

        if (DisplayParam == "All")
            FillCombo();
        // this handles both owner and plain scenarios:
        int _CurrentOwner = csaConvert.ToInt32(cboActionOwner.SelectedValue);
        DisplayDataForOwner(_CurrentOwner);
        //LoadGrid();
    }
    
    #region Boilerplate code I need on every form to handle errors and messageing

    protected void Page_PreRender(object sender, EventArgs e)
    {
        if (Message != null)
            ShowMessage(Message);

        if (Exceptions.Count != 0)
            // errors have occurred - log and display message
            LogErrors(Exceptions);
    }

    /// <summary>
    /// Handle menus specific to this page
    /// </summary>
    /// <param name="sender"></param>
    /// <param name="e"></param>
    protected void PageTitle_MenuClick(object sender, MenuClickEventArgs e)
    {
        if (e.MenuItem != null)
        {
            switch (e.MenuItem.Key)
            {
                default:
                    // stub message while stuff remains undone...
                    csaMessage _MenuMessage = new csaMessage();
                    _MenuMessage.SimpleMessage = string.Format(csaApplication.StubMessage, e.MenuItem.Text);
                    csaApplication.Message = _MenuMessage;
                    break;
            }
        }
    }

    #region Message handling

    // methods for manipulating the ucMessageDialog user control
    public void ShowMessage(string SimpleMessage)
    {
        dlgMessage.SimpleMessage = SimpleMessage;
        dlgMessage.Show();
    }

    public void ShowMessage(string Message, string Title, int Height)
    {
        dlgMessage.MessageText = Message;
        dlgMessage.MessageTitle = Title;
        dlgMessage.Height = Height;
        dlgMessage.Show();
    }

    public void ShowMessage(csaMessage message)
    {
        if (message.SimpleMessage != null)
            ShowMessage(message.SimpleMessage);
        else
            ShowMessage(message.MessageText, message.MessageTitle, message.Height);
        csaApplication.ClearMessage();
    }

    #endregion

    #region Error Handling

    public void LogErrors(List<ExceptionData> _ErrorList)
    {
        // Include relevant information from the master page - application level stuff
        // add to last item in list only (there will generally be one error, but I'm preparing for anything)
        ExceptionData Ex = _ErrorList[_ErrorList.Count - 1];

        // add local page specific details:
        Ex.AddData("Page", this.Title);

        if (CurrentUser == null)
            Ex.AddData("UserName", "No One logged in");
        else
            Ex.AddData("UserName", CurrentUser.Name);

        Ex.AddData("Version", csaApplication.Version);

        ExceptionManager.Publish(_ErrorList);

        // and let the user know whatever...
        if (Ex.InformUser)
            ShowMessage(Ex.Message);
        csaApplication.ClearErrors();
    }
    #endregion

    #endregion

    #region Grid/load Data

    private User FindOwnerByID(int OwnerID)
    {
        // scan list and return match by ID
        // to account for sillyness, return empty object if required
        User _Result = new User();
        foreach (User _Lookup in Owners)
        {
            if (_Lookup.UserID == OwnerID)
            {
                _Result = _Lookup;
                break;
            }
        }
        return _Result;
    }

    private void FillCombo()
    {
        // owner list for current project. this is buffered in State for easy/repeated access during all postbacks
        int _CurrentOwner = csaConvert.ToInt32(cboActionOwner.SelectedValue);
        cboActionOwner.DataSource = Owners;
        cboActionOwner.SelectedValue = _CurrentOwner.ToString();
    }

    private void LoadGrid()
    {
        // used only when I have to change Owner selections, otherwise the grid is automatic
        int _CurrentOwner = csaConvert.ToInt32(cboActionOwner.SelectedValue);
        if (_CurrentOwner != 0)
        {
            DisplayDataForOwner(_CurrentOwner);
            //dsProjectPlan.DataBind();
            //whgProjectPlan.DataSourceID = "";
            //whgProjectPlan.DataSourceID = "dsProjectPlan";
            whgProjectPlan.DataBind();
        }
    }

    private void DisplayDataForOwner(int OwnerID)
    {
        // do it all in one place...
        if (OwnerID == 0)
            pnlOwner.Header.Text = string.Format("Display all Actions in {0} Project Plan", csaApplication.CurrentProject.Description);
        else
            pnlOwner.Header.Text = string.Format("Show Actions/Steps owned by: {0} in Project: {1}", FindOwnerByID(OwnerID).Name, csaApplication.CurrentProject.Description);

        // Don't have to worry about defaults, as those are reset by the postback...
        //if (OwnerID != 0)
        //{
        //    pnlOwner.Expanded = false;

        //    // Update Actions query to display ONLY for a Specific owner:
        //    dsAction.SelectMethod = "GetActionsOwnerPlan";
        //    dsAction.SelectParameters.Clear();
        //    dsAction.SelectParameters.Add("Owner", System.Data.DbType.Int32, OwnerID.ToString());
        //    dsAction.SelectParameters.Add("KeyArea", System.Data.DbType.Int32, "0");
        //    dsAction.SelectParameters.Add("FunctionType", System.Data.DbType.Int32, "0");
        //    dsAction.SelectParameters.Add("StatusList", System.Data.DbType.Int32, BuildStatusList());
            
        //}
    }

    private string BuildStatusList()
    {
        // build list of all available status values:
        string _temp = "";
        foreach (Lookup _Lookup in Status)
        {
            _temp += "," + _Lookup.LookupIDString;
            //if (_Lookup.LookupID != 0)
            //    _tempDesc += ", " + _Lookup.Description;
        }
        return _temp;
    }
    protected void cmdRefresh_Click(object sender, Infragistics.WebUI.WebDataInput.ButtonEventArgs e)
    {
        // Aggressively refrsh the grid. If data was inserted in child bands, the grid isn't resorting it prperly
        // (though it does in the parent band), so th only recourse it a hard rebind...
        whgProjectPlan.DataBind();
    }

    protected void cmdSave_Click(object sender, Infragistics.WebUI.WebDataInput.ButtonEventArgs e)
    {
        // passive - used to trigger grid postbacks for save
    }

    #endregion

    #region Grid Events

    protected void whgProjectPlan_InitializeBand(object sender, Infragistics.Web.UI.GridControls.BandEventArgs e)
    {
        switch (e.Band.DataKeyFields)
        {
            case "ActionIDString":
                e.Band.Behaviors.Sorting.SortedColumns.Add("ActionNo", Infragistics.Web.UI.SortDirection.Ascending);
                break;
            case "StepIDString":
                e.Band.Behaviors.Sorting.SortedColumns.Add("StepNo", Infragistics.Web.UI.SortDirection.Ascending);
                break;
            case "ResolutionIDString":
                e.Band.Behaviors.Sorting.SortedColumns.Add("ResolutionStepNo", Infragistics.Web.UI.SortDirection.Ascending);
                break;
            default:
                break;
        }
    }
    protected void whgProjectPlan_RowsDeleting(object sender, Infragistics.Web.UI.GridControls.RowDeletingEventArgs e)
    {
        // is this fired?
        //e.Cancel = true;
        // e.Row.DataItem = instance of action, Step... according to band.
        //ShowMessage("You can't delet this row at present");
    }

    private bool ProjectView()
    {
        return (csaApplication.CurrentUser.Admin || csaApplication.CurrentProject.ProjectManager == 1);
    }

    protected void whgProjectPlan_InitializeRow(object sender, Infragistics.Web.UI.GridControls.RowEventArgs e)
    {
        // Owner view:
        if (!ProjectView())
        {
            bool Change = false;
            if (e.Row.DataItem.ToString() == "XeP3.Action")
                Change = (e.Row.Items[4].Text != csaApplication.CurrentUser.UserIDString);
            else
                Change = (e.Row.Items[5].Text != csaApplication.CurrentUser.Name);

            if (Change)
                for (int z = 0; z < e.Row.Items.Count; z++)
                    e.Row.Items[z].CssClass = "NonOwner";
        }

        // Status:
        int Col = 0;
        if (e.Row.DataItem.ToString() == "XeP3.Action")
            Col = 5;
        else
            Col = 6;

        switch (e.Row.Items[Col].Text)
        {
            case "Future":
                e.Row.Items[Col].CssClass = "StatusFuture";
                break;
            case "Start Now":
                e.Row.Items[Col].CssClass = "StatusStartNow";
                break;
            case "In Progress":
                e.Row.Items[Col].CssClass = "StatusInProgress";
                break;
            case "Completed":
                e.Row.Items[Col].CssClass = "StatusCompleted";
                break;
            case "Overdue":
                e.Row.Items[Col].CssClass = "StatusOverdue";
                break;
            default:
                // status is blank, do nothing
                break;
        }
    }

    #endregion
}
